package com.muyu.engine.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.muyu.common.core.annotation.Excel;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.engine.domain.req.RuleEngineEditReq;
import com.muyu.engine.domain.req.RuleEngineQueryReq;
import com.muyu.engine.domain.req.RuleEngineSaveReq;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

/**
 * 规则引擎对象 rule_engine
 *
 * @author muyu
 * @date 2024-05-01
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("rule_engine")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "RuleEngine", description = "规则引擎")
public class RuleEngine extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** $column.columnComment */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "id", value = "id")
    private Long id;

    /** 规则名称 */
    @Excel(name = "规则名称")
    @ApiModelProperty(name = "规则名称", value = "规则名称")
    private String name;

    /** 规则类型 */
    @Excel(name = "规则类型")
    @ApiModelProperty(name = "规则类型", value = "规则类型")
    private String type;

    /** 规则编码 */
    @Excel(name = "规则编码")
    @ApiModelProperty(name = "规则编码", value = "规则编码")
    private String code;

    /** 规则级别 */
    @Excel(name = "规则级别")
    @ApiModelProperty(name = "规则级别", value = "规则级别")
    private String level;

    /** 是否激活 */
    @Excel(name = "是否激活")
    @ApiModelProperty(name = "是否激活", value = "是否激活")
    private String isActivate;

    /** 规则状态 */
    @Excel(name = "规则状态")
    @ApiModelProperty(name = "规则状态", value = "规则状态")
    private String status;

    /** 编写文件名称 */
    @Excel(name = "编写文件名称")
    @ApiModelProperty(name = "编写文件名称", value = "编写文件名称")
    private String writeCode;

    /** 引擎编码 */
    @Excel(name = "引擎编码")
    @ApiModelProperty(name = "引擎编码", value = "引擎编码")
    private String engineCode;


    /**
    * 查询构造器
    */
    public static RuleEngine queryBuild( RuleEngineQueryReq ruleEngineQueryReq){
        return RuleEngine.builder()
                .name(ruleEngineQueryReq.getName())
                .type(ruleEngineQueryReq.getType())
                .code(ruleEngineQueryReq.getCode())
                .level(ruleEngineQueryReq.getLevel())
                .isActivate(ruleEngineQueryReq.getIsActivate())
                .status(ruleEngineQueryReq.getStatus())
                .writeCode(ruleEngineQueryReq.getWriteCode())
                .engineCode(ruleEngineQueryReq.getEngineCode())
                .build();
    }

    /**
    * 添加构造器
    */
    public static RuleEngine saveBuild(RuleEngineSaveReq ruleEngineSaveReq){
        return RuleEngine.builder()
                .name(ruleEngineSaveReq.getName())
                .type(ruleEngineSaveReq.getType())
                .code(ruleEngineSaveReq.getCode())
                .level(ruleEngineSaveReq.getLevel())
                .isActivate(ruleEngineSaveReq.getIsActivate())
                .status(ruleEngineSaveReq.getStatus())
                .writeCode(ruleEngineSaveReq.getWriteCode())
                .engineCode(ruleEngineSaveReq.getEngineCode())
                .build();
    }

    /**
    * 修改构造器
    */
    public static RuleEngine editBuild(Long id, RuleEngineEditReq ruleEngineEditReq){
        return RuleEngine.builder()
            .id(id)
                .name(ruleEngineEditReq.getName())
                .type(ruleEngineEditReq.getType())
                .code(ruleEngineEditReq.getCode())
                .level(ruleEngineEditReq.getLevel())
                .isActivate(ruleEngineEditReq.getIsActivate())
                .status(ruleEngineEditReq.getStatus())
                .writeCode(ruleEngineEditReq.getWriteCode())
                .engineCode(ruleEngineEditReq.getEngineCode())
                .build();
    }

}
