package com.muyu.engine.service.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.engine.domain.RuleEngines;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import com.muyu.engine.mapper.RuleEngineMapper;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.service.RuleEngineService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 规则引擎Service业务层处理
 *
 * @author muyu
 * @date 2024-05-01
 */
@Slf4j
@Service
public class RuleEngineServiceImpl extends ServiceImpl<RuleEngineMapper, RuleEngine>  implements RuleEngineService {


      //  全部变量
      public static String url = "D:\\JavaTest\\";

    /**
     * 查询规则引擎列表
     *
     * @param ruleEngine 规则引擎
     * @return 规则引擎
     */
    @Override
    public List<RuleEngine> list(RuleEngine ruleEngine) {
        LambdaQueryWrapper<RuleEngine> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleEngine.getName())){
            queryWrapper.like(RuleEngine::getName, ruleEngine.getName());
        }

        if (ObjUtils.notNull(ruleEngine.getType())){
            queryWrapper.eq(RuleEngine::getType, ruleEngine.getType());
        }

        if (ObjUtils.notNull(ruleEngine.getCode())){
            queryWrapper.eq(RuleEngine::getCode, ruleEngine.getCode());
        }

        if (ObjUtils.notNull(ruleEngine.getLevel())){
            queryWrapper.eq(RuleEngine::getLevel, ruleEngine.getLevel());
        }

        if (ObjUtils.notNull(ruleEngine.getIsActivate())){
            queryWrapper.eq(RuleEngine::getIsActivate, ruleEngine.getIsActivate());
        }

        if (ObjUtils.notNull(ruleEngine.getStatus())){
            queryWrapper.eq(RuleEngine::getStatus, ruleEngine.getStatus());
        }

        return list(queryWrapper);
    }

    @Override
    public String code(RuleEngine ruleEngine) throws Exception {

        if (ruleEngine.getType().equals("rule-custom")) {
             this.setParams(ruleEngine);
        }
        return "修改成功";
    }

    @Override
    public void add(RuleEngine saveBuild) throws Exception {
        this.save(saveBuild);
        if (saveBuild.getType().equals("rule-custom")){
            this.setParams(saveBuild);
        }
    }

    @Override
    public String load(RuleEngine ruleEngine) {
        try {
            //  Files.exists(Paths.get("D:\\JavaTest\\" + ruleEngine.getWriteCode() + ".java"));
           //找到文件并读取
            Path path = Paths.get(url + ruleEngine.getWriteCode() + ".java");
            if (Files.exists(path)) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(Files.newInputStream(path), StandardCharsets.UTF_8));
                String line;
                StringBuilder content = new StringBuilder();
                while ((line = reader.readLine()) != null) {
                    content.append(line).append("\n");
                }
                reader.close();
                return content.toString();
            } else {
                return "文件不存在";
            }
        } catch (Exception ex) {
            return Result.error(ex).getMsg();
        }
    }

    @Override
    public String writeCodeAdd(RuleEngines ruleEngine) throws IOException {
        String aa = this.aa(ruleEngine, ruleEngine.getSql());
        return aa;
    }

    @Override
    public void updateRuleEngineVersionIsActivate(RuleEngine ruleEngine) {
        log.info("开始修改");
        if (ruleEngine.getIsActivate().equals("activated")){
            RuleEngine build = RuleEngine.builder()
                    .isActivate("no-activate")
                    .build();
            this.update(build,new LambdaQueryWrapper<RuleEngine>().eq(RuleEngine::getId,ruleEngine.getId()));
       log.info("修改成功",build);
        }else
        if (ruleEngine.getIsActivate().equals("no-activate")){
            RuleEngine build = RuleEngine.builder()
                    .isActivate("activated")
                    .build();
            this.update(build,new LambdaQueryWrapper<RuleEngine>().eq(RuleEngine::getId,ruleEngine.getId()));
            log.info("修改成功",build);
        }
    }

    //修改规则引擎状态
    @Override
    public void updateRuleEngineVersionStates(RuleEngine ruleEngine) {
        if (ruleEngine.getStatus().equals("0")){
            RuleEngine build = RuleEngine.builder()
                    .status("1")
                    .build();
            this.update(build,new LambdaQueryWrapper<RuleEngine>().eq(RuleEngine::getId,ruleEngine.getId()));
            log.info("修改成功",build);
        }else
        if (ruleEngine.getStatus().equals("1")){
            RuleEngine build = RuleEngine.builder()
                    .status("0")
                    .build();
            this.update(build,new LambdaQueryWrapper<RuleEngine>().eq(RuleEngine::getId,ruleEngine.getId()));
            log.info("修改成功",build);
        }
    }

    public String setParams (RuleEngine ruleEngine) throws Exception {

        String  sql ="\n" +
                "/**\n" +
                " * @Author: yangle\n" +
                " * @date: 2024/5/3\n" +
                " */\n" +
                "public class "+ruleEngine.getWriteCode()+" {\n" +
                "\n" +
                "  public static void main(String[] args) {\n" +
                "           String s=\"sss\";\n" +
                "        if (s.equals(\"sss\")){\n" +
                "            System.out.println(true);\n" +
                "        }else {\n" +
                "            System.out.println(false);\n" +
                "        }\n" +
                "    }\n" +
                "}\n" +
                "  ";
       return this.aVoid(ruleEngine,sql);

    }

    public String  aVoid (RuleEngine ruleEngine,String sql) {
        try {
            // 文件路径
            String fileName = ruleEngine.getWriteCode() + ".java";
            String filePath =url + fileName;


            // 编译Java文件
            ProcessBuilder pbCompile = new ProcessBuilder("javac",fileName);
            pbCompile.directory(Paths.get(url).toFile()); // 设置工作目录
            pbCompile.redirectErrorStream(true);

            // 将字符串写入文件
            Path path = Paths.get(filePath);
            Files.write(path, sql.getBytes(StandardCharsets.UTF_8));

            // 编译
            Process compileProcess = pbCompile.start();
            // 等待编译结果
            compileProcess.waitFor();
            // 编译结果
            int compileResult = compileProcess.exitValue();
            System.out.println(compileResult == 0 ? "编译成功" : "编译失败");

            ProcessBuilder pbRun = new ProcessBuilder("java",ruleEngine.getWriteCode());
            pbRun.directory(Paths.get(url).toFile());
            Process runProcess = pbRun.start();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(runProcess.getInputStream()));
            String str;
            str = bufferedReader.readLine();
            while (str != null) {
                System.out.println(str);
                break;
            }
            runProcess.waitFor();
            int runResult = runProcess.exitValue();
            System.out.println(runResult);

            System.out.println(runResult == 0 ? "运行成功" : "运行失败");

            return (compileResult == 0 ? "编译成功" : "编译失败");
        }catch (Exception e){
            return Result.error(e).getMsg();
        }


    }
    public String  aa (RuleEngines ruleEngine,String sql) {
        try {
            // 文件路径
            String fileName = ruleEngine.getWriteCode() + ".java";
            String filePath =url + fileName;

            // 编译Java文件
            ProcessBuilder pbCompile = new ProcessBuilder("javac",fileName);
            pbCompile.directory(Paths.get(url).toFile()); // 设置工作目录
            pbCompile.redirectErrorStream(true);

            // 将字符串写入文件
            Path path = Paths.get(filePath);
            Files.write(path, sql.getBytes(StandardCharsets.UTF_8));

            // 编译
            Process compileProcess = pbCompile.start();
            // 等待编译结果
            compileProcess.waitFor();
            // 编译结果
            int compileResult = compileProcess.exitValue();
            System.out.println(compileResult  == 0 ? "编译成功" : "编译失败");

            ProcessBuilder pbRun = new ProcessBuilder("java",ruleEngine.getWriteCode());
            pbRun.directory(Paths.get(url).toFile());
            Process runProcess = pbRun.start();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(runProcess.getInputStream()));
            String str;
            str = bufferedReader.readLine();
            while (str != null) {
                System.out.println(str);
                break;
            }
            runProcess.waitFor();
            int runResult = runProcess.exitValue();
            System.out.println(runResult);

            System.out.println(runResult == 0 ? "运行成功" : "运行失败");

            return (compileResult == 0 ? "编译成功" : "编译失败");

        }catch (Exception e){
            return Result.error(e).getMsg();
        }

    }
}
