package com.muyu.engine.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.domain.rule_engine_version.RuleEngineList;
import com.muyu.engine.domain.rule_engine_version.RuleEngineVersion;
import com.muyu.engine.mapper.RuleEngineMapper;
import com.muyu.engine.mapper.RuleEngineVersionMapper;
import com.muyu.engine.service.RuleEngineService;
import com.muyu.engine.service.RuleEngineVersionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.tools.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 规则引擎版本业务实现层 RuleEngineVersionServiceImpl
 *
 * @author LeYang
 * on 2024/5/6
 */
@Slf4j
@Service
public class RuleEngineVersionServiceImpl extends ServiceImpl<RuleEngineVersionMapper, RuleEngineVersion> implements RuleEngineVersionService  {


    @Override
    public List<RuleEngineVersion> list(RuleEngineVersion ruleEngineVersion) {
        LambdaQueryWrapper<RuleEngineVersion> queryWrapper = new LambdaQueryWrapper<>();


        if (Objects.nonNull(ruleEngineVersion.getRuleEngineId())){
            queryWrapper.eq(RuleEngineVersion::getRuleEngineId,ruleEngineVersion.getRuleEngineId());
        }
        if (Objects.nonNull(ruleEngineVersion.getCode())){
            queryWrapper.eq(RuleEngineVersion::getCode,ruleEngineVersion.getCode());
        }
        if (Objects.nonNull(ruleEngineVersion.getName())){
            queryWrapper.eq(RuleEngineVersion::getName,ruleEngineVersion.getName());
        }
        if (Objects.nonNull(ruleEngineVersion.getVersionCode())){
            queryWrapper.eq(RuleEngineVersion::getVersionCode,ruleEngineVersion.getVersionCode());
        }
        if (Objects.nonNull(ruleEngineVersion.getWriteCode())){
            queryWrapper.eq(RuleEngineVersion::getWriteCode,ruleEngineVersion.getWriteCode());
        }
        if (Objects.nonNull(ruleEngineVersion.getStatus())){
            queryWrapper.eq(RuleEngineVersion::getStatus,ruleEngineVersion.getStatus());
        }
        if (Objects.nonNull(ruleEngineVersion.getIsActivate())){
            queryWrapper.eq(RuleEngineVersion::getIsActivate,ruleEngineVersion.getIsActivate());
        }

        return list(queryWrapper);
    }
@Autowired
private RuleEngineService ruleEngineService;
    @Override
    public RuleEngineList listResult(Integer id) {
        log.info("开始查询");
        LambdaQueryWrapper<RuleEngineVersion> queryWrapper = new LambdaQueryWrapper<>();
        //根据id 查询规则
        queryWrapper.eq(RuleEngineVersion::getRuleEngineId,id);
        List<RuleEngineVersion> list = list(queryWrapper);

        LambdaQueryWrapper<RuleEngine> wrapper = new LambdaQueryWrapper<>();
        //根据id 查询规则引擎对象
        wrapper.eq(RuleEngine::getId,id);
        List<RuleEngine> ruleEngineList = ruleEngineService.list(wrapper);
        RuleEngineList collect = ruleEngineList.stream()
                .map(ruleEngine ->
                        RuleEngineList.builder()
                                .id(ruleEngine.getId())
                                .name(ruleEngine.getName())
                                .type(ruleEngine.getType())
                                .code(ruleEngine.getCode())
                                .level(ruleEngine.getLevel())
                                .isActivate(ruleEngine.getIsActivate())
                                .status(ruleEngine.getStatus())
                                .writeCode(ruleEngine.getWriteCode())
                                .engineCode(ruleEngine.getEngineCode())
                                .ruleEngineVersionList(list)
                                .build()
                        ).collect(Collectors.toList()).get(0);


        return collect;
    }

    //全局变量
    public static String url="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\src\\main\\java\\com\\muyu\\engine\\custom\\";
    @Override
    public void RuleEngineVersionAdd(RuleEngineVersion ruleEngineVersion) {
          this.save(ruleEngineVersion);
    }

    @Override
    public void updataEngineVersionStatus(RuleEngineVersion ruleEngineVersion) {

        RuleEngineVersion build = RuleEngineVersion.builder()
                .status("1")
                .codeIng(ruleEngineVersion.getCodeIng())
                .build();
        this.update(build,new LambdaQueryWrapper<RuleEngineVersion>().eq(RuleEngineVersion::getId,ruleEngineVersion.getId()));
        this.writeCodeAdd(ruleEngineVersion);
        log.info("编码保存成功",build);

    }

    //生产源文件
    public void writeCodeAdd(RuleEngineVersion ruleEngineVersion) {
        try{
            //文件路径
           String fileName =ruleEngineVersion.getVersionCode()+".java";
           String filePath = url+fileName;

           //编写java文件
            ProcessBuilder pbCompiler = new ProcessBuilder("javac", fileName);
            pbCompiler.directory(Paths.get(url).toFile());
            pbCompiler.redirectErrorStream(true);

            //将字符串写入文件中
            Path path = Paths.get(filePath);
            Files.write(path,ruleEngineVersion.getCodeIng().getBytes(StandardCharsets.UTF_8));

            // 编译
                Process process = pbCompiler.start();

                BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            // 等待编译结果
                int exitCode = process.waitFor();

                if (exitCode == 0) {
                    System.out.println("编译成功");
                } else {
                    System.out.println("编译失败");
                }

                //编译成class文件
            ProcessBuilder processBuilder = new ProcessBuilder("java", ruleEngineVersion.getWriteCode());
               processBuilder.directory(Paths.get(url).toFile());
               processBuilder.redirectErrorStream(true);
               Process Runprocess = processBuilder.start();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(Runprocess.getInputStream()));
            String str ;
            while ((str = bufferedReader.readLine()) != null) {
                System.out.println(str);
                break;
            }
            Runprocess.waitFor();
            int i = Runprocess.exitValue();
            if (i==0){
                System.out.println("运行成功");
            }else {
                System.out.println("运行失败");
            }
        }catch (Exception e){
             throw new RuntimeException();
        }

    }

//    private String url ="D:\\work\\zglkh\\cloud-server\\muyu-modules\\muyu-rule-engine\\muyu-rule-engine-common\\src\\main\\java\\com\\muyu\\engine\\java";
//
//    //生产源文件
//    public void writeCodeAdd(RuleEngineVersion ruleEngineVersion) {
//        try {
//
//            String className = ruleEngineVersion.getVersionCode();
//            String content = ruleEngineVersion.getCodeIng();
//            // 指定输出目录
//            JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
//
//            try (StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, null)) {
//                // 创建一个内存中的源文件
//                JavaFileObject sourceFileObject = new JavaSourceFromString(className, content);
//
//                // 编译选项
//                Iterable<String> options = Arrays.asList("-d", url);
//                // 编译源代码
//                JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null, options, null, Arrays.asList(sourceFileObject));
//                boolean success = task.call();
//
//                if (success) {
//                    System.out.println("编译成功");
//                } else {
//                    System.out.println("编译失败");
//                }
//
//
//            } catch (Exception e) {
//                e.printStackTrace();
//            }
//        }catch (Exception e) {
//            e.printStackTrace();
//        }
//
//    }
//    static class JavaSourceFromString extends SimpleJavaFileObject {
//        final String code;
//
//
//        JavaSourceFromString(String name, String code) {
//            super(URI.create("string:///" + name.replace('.', '/') + Kind.SOURCE.extension), Kind.SOURCE);
//            this.code = code;
//        }
//        @Override
//        public CharSequence getCharContent(boolean ignoreEncodingErrors) {
//            return code;
//        }
//    }
}
